local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
if not HeadCount then
	print("Failed to load Boss Detection, we cant find HeadCount")
	return
end

local plugin = HeadCount:NewModule("BossDetection","AceEvent-3.0","AceTimer-3.0")
local BB = LibStub("LibBabble-Boss-3.0"):GetUnstrictLookupTable()

local CallbackHandler = LibStub:GetLibrary("CallbackHandler-1.0")

plugin.Callbacks = CallbackHandler:New(plugin)

local UnitExists = UnitExists
local UnitAffectingCombat = UnitAffectingCombat
local UnitName = UnitName
local UnitClassification = UnitClassification
local InstanceBossList = {}
local KilledBossList = {}
local InstanceIndex = 0
local monitoring = false
local engagedBossName = nil
local engagedBosses = {}
local targetTable = {"target", "targettarget", "focus", "focustarget", "mouseover", "mouseovertarget"}
local BossIsEngaged = false
local rriHandle = nil
local scanHandle = nil

function plugin:OnInitialize()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA","CheckInstance")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","CheckInstance")
	self:RegisterEvent("RAID_ROSTER_UPDATE","CheckInstance")
	self:RegisterEvent("UPDATE_INSTANCE_INFO","CheckFlexLocks")
	self:BuildTargetTable()
end

function plugin:OnEnabled()
	-- Lets check our data right now
end

function plugin:CheckInstance(event)
	-- whenever we change zones, or enter an instance restart monitoring
	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then
		self:StopMonitoring()
	end
	-- Otherwise once in a raid/party
	if GetNumRaidMembers() > 0 or GetNumPartyMembers() > 0 then
		RequestRaidInfo()
		self:StartMonitoring()
	else
		self:StopMonitoring()
	end
end

function plugin:GetInstanceBossList()
	if IsInInstance() then
		local lockTimeleft, isPreviousInstance, encountersTotal, encountersComplete = GetInstanceLockTimeRemaining();
		for i=1,encountersTotal do
			local bossName, texture, isKilled = GetInstanceLockTimeRemainingEncounter(i);
			InstanceBossList[bossName] = i
			KilledBossList[bossName] = false
		end
	end
end

function plugin:IsValidBossName(name)
	return InstanceBossList[name]
end

function plugin:GetSavedInstanceData()
	if InstanceIndex == 0 then
		for k,v in pairs(GetFullRaidList()) do
			local 	name, id, reset, difficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(k)
			local  bossName,text,isKilled = GetSavedInstanceEncounterInfo(k,1)
			if InstanceBossList[bossName] then
				InstanceIndex = k
				-- Get the status of the active raid kill list
				for m=1,numEncounters do
					bossName,text,isKilled = GetSavedInstanceEncounterInfo(k,m)
					KilledBossList[bossName] = isKilled
				end
			end
		end
	else
		local 	name, id, reset, difficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(InstanceIndex)
		for m=1,numEncounters do
			local bossName,text,isKilled = GetSavedInstanceEncounterInfo(InstanceIndex,m)
			KilledBossList[bossName] = isKilled
		end
	end
end

function plugin:StartMonitoring()
	if monitoring then return end
	-- Get the list of instance bosses
	if IsInInstance() then
		self:GetInstanceBossList()
		self:GetSavedInstanceData()
	end
	monitoring = true
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "ExitedCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteredCombat")
	self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT","EnteredCombat")
end

function plugin:StopMonitoring()
	monitoring = false
	table.wipe(InstanceBossList)
	table.wipe(KilledBossList)
	table.wipe(engagedBosses)
	engagedBossName = nil
	InstanceIndex = 0
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
end

function plugin:BuildTargetTable()
	for i = 1, 4 do targetTable[#targetTable+1] = string.format("boss%d", i) end
	for i = 1, 4 do targetTable[#targetTable+1] = string.format("party%dtarget", i) end
	for i = 1, 40 do targetTable[#targetTable+1] = string.format("raid%dtarget", i) end
end

local fBosses = {}
local function FindBossUnits()
	table.wipe(fBosses)
	for i, unit in ipairs(targetTable) do
		local name = UnitName(unit)
		if UnitExists(unit) and not UnitIsPlayer(unit) and UnitClassification(unit) == "worldboss"then
			local lName = BB[name]
			if lName then
				fBosses[lName] = true
			else
				--HeadCount:LogWarning("Boss "..name.." missing locaization")
				local guid = UnitGUID(unit)
				guid = tonumber(guid:sub(7, 10), 16)
				if not HeadCount.BOSS_BLACKLIST[guid..""] then
					fBosses[name] = true
				end
			end
		end
		if UnitExists(unit) and not UnitIsPlayer(unit) and UnitClassification(unit) == "elite" then
			local lName = BB[name]
			if lName and HeadCount.BOSS_WHITELIST[lName] then
				fBosses[lName] = true
			end
			-- On non boss units, check the alias list and add them in if so determined
			if lName and HeadCount.BOSS_ALIASES[lName] then
				fBosses[lName] = true
			end
		end
	end
	for k,v in pairs(fBosses) do
		if not HeadCount.BOSS_BLACKLIST[k] then
			tinsert(engagedBosses,k)
		end
	end
end
local function GetBossUnit(bossName)
	for i, unit in ipairs(targetTable) do
		if UnitExists(unit) and not UnitIsPlayer(unit) then
			local uName = UnitName(unit)
			if uName == bossName then
				return unit
			end
		end
	end
end

function plugin:Scan()
	-- Find all WorldBossUnits
	table.wipe(engagedBosses)
	FindBossUnits()
	local engaged = false
	for k,v in ipairs(engagedBosses) do
		local unit = GetBossUnit(v)
		if unit and UnitAffectingCombat(unit) then
			engaged = true
		end
	end
	return engaged
end

function plugin:ScanForEngage()
	if self:Scan() and not BossEngaged then
		BossIsEngaged = true
		self:DetermineEncounter()
	else
		scanHandle = self:ScheduleTimer(plugin.ScanForEngage,0.5,self)
	end
end

function plugin:ScanForWipe()
	if not self:Scan() then
		-- No units in combat anymore, lets see if the raid lock updated.
		if IsInInstance() then
			-- Lets check raid locks
			RequestRaidInfo()
		else
			-- World boss, lets see if he is dead.
			local unit = GetBossUnit(engagedBossName)
			if not unit then return end
			if UnitIsDead(unit) then
				-- Success
				self.Callbacks:Fire("BossKilled",engagedBossName)
				engagedBossName = nil
				BossIsEngaged = false
			else
				-- Wipe!
				self.Callbacks:Fire("BossWipe",engagedBossName)
				engagedBossName = nil
				BossIsEngaged = false
			end
		end
	else
		self:ScheduleTimer(plugin.ScanForWipe,0.5,self)
	end
end


function plugin:DetermineEncounter()
	if engagedBossName then return end
	local foundEncounter = false
	local bossCount = 0
	-- Only 1 boss unit present
	if #engagedBosses == 1 then
		engagedBossName = engagedBosses[1]
		if BB[engagedBossName] then
			engagedBossName = BB[engagedBossName]
		end
	end
	-- Multiple Boss Units present, find the alias
	for k,v in pairs(engagedBosses) do
		local localizedBoss = BB[v]
		if localizedBoss then
			if HeadCount.BOSS_ALIASES[localizedBoss] then
				engagedBossName = HeadCount.BOSS_ALIASES[localizedBoss]
			end
		end
	end

	if engagedBossName and HeadCount.BOSS_ALIASES[engagedBossName] then
		engagedBossName = HeadCount.BOSS_ALIASES[engagedBossName]
	end

	-- We didnt have an alias for this encounter and it isnt a single boss
	if not engagedBossName then
		HeadCount:LogWarning(string.format("No Alias found for this multiboss encounter! Please report this to HeadCount2 project %s",table.concat(engagedBosses, ",")))
	else
		KilledBossList[engagedBossName] = false
		self.Callbacks:Fire("BossEngaged",engagedBossName)
	end
end

function plugin:ExitedCombat()
	self:CancelTimer(scanHandle,true)
	self:ScanForWipe()
end

function plugin:EnteredCombat()
	table.wipe(engagedBosses)
	self:ScanForEngage()
end

local tmpBossList = {}

function plugin:CheckFlexLocks()
	if not monitoring then return end
	if not engagedBossName then return 	end
	if BossIsEngaged then return end
	if rriHandle then rriHandle = nil end

	table.wipe(tmpBossList)
	for k,v in pairs(KilledBossList) do
		tmpBossList[k] = v
	end
	self:GetSavedInstanceData()
	for k,v in pairs(KilledBossList) do
		if v ~= tmpBossList[k] and engagedBossName then
			-- We have an updated lock
			if BB[v] and BB[v] ~= engagedBossName then
				if HeadCount.BOSS_ALIASES[BB[v]] then
					engagedBossName = HeadCount.BOSS_ALIASES[BB[v]]
				else
					engagedBossName = BB[v]
				end
			end
			self.Callbacks:Fire("BossKilled",engagedBossName)
			engagedBossName = nil
		end
	end
	-- Boss wasnt in the updated lock, lets see if we wiped
	if engagedBossName and not BossIsEngaged then
		local unit = GetBossUnit(engagedBossName)
		if #engagedBosses > 1 then
			for k,v in pairs(engagedBosses) do
				unit = GetBossUnit(v)
				if unit and not UnitIsDead(unit) then
					self.Callbacks:Fire("BossWipe",engagedBossName)
					engagedBossName = nil
					BossIsEngaged = false
				end
			end
		else
			if unit and not UnitIsDead(unit) then
				-- Wipe!
				self.Callbacks:Fire("BossWipe",engagedBossName)
				engagedBossName = nil
				BossIsEngaged = false
			end
		end
	end
end