local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
local GUI = HeadCount:NewModule("GUI")
if not HeadCount then return end

local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")
local AGU = LibStub("AceGUI-3.0")
local MainWindow = nil


local SimpleGroupBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
}


local function ShowMultilineTooltip(widget)
	GameTooltip:SetOwner(widget.frame,"ANCHOR_CURSOR", -6, 0)
	local data = widget:GetUserData("tooltip.multiline.data")
	for k,v in pairs(data) do
		GameTooltip:AddLine(v)
	end
	GameTooltip:Show()
end

local function ShowTooltip(widget)
	GameTooltip:SetOwner(widget.frame,"ANCHOR_CURSOR", -6, 0)
	GameTooltip:AddLine(widget:GetUserData("tooltip.key"),0.63, 0.63, 1.0)
	if widget:GetUserData("tooltip.key") == L["gui.annouce.title"] then
		local raidTracker = HeadCount:GetModule("RaidTracker")
		local isWaitlistAcceptanceEnabled = raidTracker.isWaitlistAcceptanceEnabled
		if isWaitlistAcceptanceEnabled then
			GameTooltip:AddLine(L["gui.announce.wait.list"] .. ": " .. L["Active"], 1.0, 1.0, 1.0)
		else
			GameTooltip:AddLine(L["gui.announce.wait.list"] .. ": " .. L["Inactive"], 1.0, 1.0, 1.0)
		end
	end
	GameTooltip:Show()
end
local function HideTooltip()
	GameTooltip:Hide()
end

local function SetupMultilineTooltip(widget,table)
	widget:SetUserData("tooltip.multiline.data",table)
	widget:SetCallback("OnEnter",ShowMultilineTooltip)
	widget:SetCallback("OnLeave",HideTooltip)

end

local function SetupTooltip(widget,tooltip)
	widget:SetUserData("tooltip.key",tooltip)
	widget:SetCallback("OnEnter",ShowTooltip)
	widget:SetCallback("OnLeave",HideTooltip)
end

function GUI:ShowLootPopup(raidId,lootId)
	if GUI.lootwindow then return end
	self = GUI
	local frame = AGU:Create("HeadCountDialog")
	frame:SetTitle(L["Manage loot"])
	frame:SetWidth(400)
	frame:SetHeight(350)
	frame:SetCallback("OnClose", function(widget)
		frame.lbl1.label:SetJustifyH("LEFT")
		frame.lbl1 = nil
		frame.lbl2.label:SetJustifyH("LEFT")
		frame.lbl2 = nil
		widget:ReleaseChildren()
		widget:Release()
		GUI.lootwindow = nil
	end)

	frame:SetCallback("OnAccept", function(widget)
		local looter = frame:GetUserData("looter-choice")
		local source = frame:GetUserData("source-choice")
		local cost   = frame:GetUserData("cost-choice")
		local note   = frame:GetUserData("note-choice")
		if not note then note = "" end
		if not cost then cost = "0" end

		if looter and source and cost then
			local cost = tonumber(cost)
			if note then
				note = HeadCount:trim(note)	-- trim whitespace
			end
			local Tracker = HeadCount:GetModule("RaidTracker")
			Tracker:lootManagementUpdate(raidId, lootId, looter, source, cost, note)
		else
			if not looter then
				HeadCount:LogInformation(string.format(L["warning.loot.manage.looter"], HeadCount.TITLE, HeadCount.VERSION))
			end

			if not source then
				HeadCount:LogInformation(string.format(L["warning.loot.manage.source"], HeadCount.TITLE, HeadCount.VERSION))
			end

			if not cost then
				HeadCount:LogInformation(string.format(L["warning.loot.manage.cost"], HeadCount.TITLE, HeadCount.VERSION))
			end
		end
		widget:ReleaseChildren()
		widget:Release()
		GUI.lootwindow = nil
		if GUI.ActiveFrame and GUI.ActiveFrame == "LOOT" then
			GUI:ShowLootInRaid()
		end
	end)

	local raid = HeadCount:GetModule("RaidTracker").raidListWrapper:getRaidById(raidId)
	if not raid then
		return
	end

	local loot = raid.lootList[lootId]

	frame:SetUserData("looter-choice",loot.player)
	frame:SetUserData("source-choice",loot.source)
	frame:SetUserData("cost-choice",tonumber(loot.cost))
	frame:SetUserData("note-choice",loot.note)


	frame:SetAcceptLabel(L["Save"])
	frame:SetCancelLabel(L["Cancel"])
	--frame:EnableHotKey()

	frame:SetLayout("Flow")
	self.lootwindow = frame

	local raidDesc = AGU:Create("Label")
	local raidDescription = HeadCount:getDateTimeAsString(raid:retrieveStartingTime())
	local zone = raid.zone
	if zone then
		-- raid zone exists
		raidDescription = raidDescription .. " - " .. zone
	else
		raidDescription = raidDescription .. " - " .. UNKNOWN
	end
	raidDesc:SetText(raidDescription)
	raidDesc:SetRelativeWidth(1)
	raidDesc.label:SetJustifyH("MIDDLE")
	frame.lbl1 = raidDesc
	frame:AddChild(raidDesc)

	local rowSpacer = AGU:Create("Label")
	rowSpacer:SetText("  ")
	rowSpacer:SetRelativeWidth(1)
	frame:AddChild(rowSpacer)

	-- Create Loot Link
	local linkLabel = AGU:Create("Label")
	linkLabel:SetText(loot.link)
	linkLabel:SetRelativeWidth(1)
	linkLabel.label:SetJustifyH("MIDDLE")
	frame:AddChild(linkLabel)
	frame.lbl2 = linkLabel

	-- Create Looter Dropdown
	local lootDrop = AGU:Create("Dropdown")
	lootDrop:SetLabel(L["gui.loot.lootplayer"])
	lootDrop.list = {}
	-- Add teh looters colored by class
	local playerList = raid:retrievePlayerListByClass()

	for index,clazz in pairs(CLASS_SORT_ORDER) do
		for mindex,player in pairs(playerList[clazz]) do
			local color = RAID_CLASS_COLORS[clazz]
			local colored = string.format("\124cff%.2x%.2x%.2x", color.r*255, color.g*255, color.b*255)..player.."\124r"
			lootDrop:AddItem(player,colored)
		end
	end
	for index,player in pairs(playerList["UNKNOWN"]) do
		local colored = string.format("|c9E9E9EFF%s|r",player)
		lootDrop:AddItem(player,colored)
	end
	local cr,cg,cb = 1.0,0.82,0
	-- Add the special looters

	lootDrop:AddItem(L["Disenchanted"],string.format("|cffD4A017%s|r",L["Disenchanted"]))
	lootDrop:AddItem(L["Bank"],string.format("|cffD4A017%s|r",L["Bank"]))
	lootDrop:AddItem(L["Offspec"],string.format("|cffD4A017%s|r",L["Offspec"]))
	lootDrop:SetValue(loot.player)
	lootDrop:SetRelativeWidth(0.5)

	lootDrop:SetCallback("OnValueChanged", function(self,event, key,checked)
		frame:SetUserData("looter-choice",key)
	end)

	local lsp1 = AGU:Create("Label")
	lsp1:SetText("   ")
	lsp1:SetRelativeWidth(0.20)
	local bankLabel = AGU:Create("InteractiveLabel")
	bankLabel:SetImage("Interface\\Icons\\INV_Misc_Bag_26_Spellfire")
	bankLabel:SetImageSize(18,18)
	bankLabel:SetRelativeWidth(0.07)
	bankLabel:SetCallback("OnClick", function()
		lootDrop:SetValue(L["Bank"])
		frame:SetUserData("looter-choice",L["Bank"])
	end)
	SetupTooltip(bankLabel,L["gui.loot.bank.tooltip"])

	local deLabel = AGU:Create("InteractiveLabel")
	deLabel:SetImage("Interface\\Icons\\INV_Enchant_Disenchant")
	deLabel:SetImageSize(18,18)
	deLabel:SetRelativeWidth(0.05)
	deLabel:SetCallback("OnClick", function()
		lootDrop:SetValue(L["Disenchanted"])
		frame:SetUserData("looter-choice",L["Disenchanted"])
	end)
	SetupTooltip(deLabel,L["gui.loot.de.tooltip"])

	local offspecLabel = AGU:Create("InteractiveLabel")
	offspecLabel:SetImage("Interface\\Icons\\INV_Misc_Gift_05")
	offspecLabel:SetImageSize(18,18)
	offspecLabel:SetRelativeWidth(0.05)
	offspecLabel:SetCallback("OnClick", function()
		lootDrop:SetValue(L["Offspec"])
		frame:SetUserData("looter-choice",L["Offspec"])
	end)
	SetupTooltip(offspecLabel,L["gui.loot.offspec.tooltip"])

	local esp1 = AGU:Create("Label")
	esp1:SetText("   ")
	esp1:SetRelativeWidth(0.10)
	frame:AddChild(lsp1)
	frame:AddChild(lootDrop)
	frame:AddChild(bankLabel)
	frame:AddChild(deLabel)
	frame:AddChild(offspecLabel)
	frame:AddChild(esp1)


	-- Add the source dropdown
	local sourceDrop = AGU:Create("Dropdown")
	sourceDrop:SetLabel(L["gui.loot.lootsource"])
	sourceDrop.list = {}

	local bossList = raid:retrieveOrderedBossList()
	for index, boss in pairs(bossList) do
		if (strlen(boss.name) > 0) then
			sourceDrop:AddItem(boss.name,boss.name)
		end
	end
	sourceDrop:AddItem(HeadCount.DEFAULT_LOOT_SOURCE,HeadCount.DEFAULT_LOOT_SOURCE)
	sourceDrop:SetRelativeWidth(0.5)
	sourceDrop:SetValue(loot.source)
	sourceDrop:SetCallback("OnValueChanged", function(self,event, key,checked)
		frame:SetUserData("source-choice",key)
	end)

	local lsp2 = AGU:Create("Label")
	lsp2:SetText("   ")
	lsp2:SetRelativeWidth(0.20)
	local esp2 = AGU:Create("Label")
	esp2:SetText("   ")
	esp2:SetRelativeWidth(0.3)

	frame:AddChild(lsp2)
	frame:AddChild(sourceDrop)
	frame:AddChild(esp2)

	-- Add the Loot cost input

	local lsp3 = AGU:Create("Label")
	lsp3:SetText("   ")
	lsp3:SetRelativeWidth(0.20)

	local lootCost = AGU:Create("EditBox")
	lootCost:SetLabel(L["gui.loot.lootcost"])
	lootCost:SetText(tonumber(loot.cost))
	lootCost:SetCallback("OnTextChanged", function (self,event, text)
		frame:SetUserData("cost-choice",text)
	end)
	lootCost:SetRelativeWidth(0.5)
	local esp3 = AGU:Create("Label")
	esp3:SetText("   ")
	esp3:SetRelativeWidth(0.3)

	frame:AddChild(lsp3)
	frame:AddChild(lootCost)
	frame:AddChild(esp3)

	-- Add the Loot note input

	local lsp4 = AGU:Create("Label")
	lsp4:SetText("   ")
	lsp4:SetRelativeWidth(0.20)

	local lootNote = AGU:Create("EditBox")
	lootNote:SetLabel(L["gui.loot.lootnote"])
	lootNote:SetText(loot.note)
	lootNote:SetCallback("OnTextChanged", function (self,event, text)
		frame:SetUserData("note-choice",text)
	end)
	lootNote:SetRelativeWidth(0.5)
	local esp4 = AGU:Create("Label")
	esp4:SetText("   ")
	esp4:SetRelativeWidth(0.3)

	frame:AddChild(lsp4)
	frame:AddChild(lootNote)
	frame:AddChild(esp4)
end


function GUI:Refresh(reset)
	self:UpdateRaidHistoryLabel()
	if self.annouce_window then
		AGU:Release(self.annouce_window)
		self.annouce_window = nil
	end
	if self.lootwindow then
		AGU:Release(self.lootwindow)
		self.lootwindow = nil
	end
	self:PopulateRaidList()
end

function GUI:RefreshActivePanel()
	if GUI.ActiveFrame == "PLAYER" and MainWindow then
		GUI:ShowPlayersInRaid()
	end
	if GUI.ActiveFrame == "LOOT" and MainWindow then
		GUI:ShowLootInRaid()
	end
	if GUI.ActiveFrame == "BOSS" and MainWindow then
		GUI:ShowBossListInRaid()
	end
	if GUI.ActiveFrame == "WAIT" and MainWindow then
		GUI:ShowWaitListInRaid()
	end
end

function GUI:UpdateRaidHistoryLabel()
	if MainWindow then
		local raidCount = HeadCount.db.profile.raidListWrapper.numberOfRaids
		MainWindow.history_label:SetText(string.format(L["gui.history.title"],raidCount))
	end
end

local function RemoveAllRaids()
	HeadCount:GetModule("RaidTracker"):removeAllRaids()
	GUI.selectedRaid = nil
	GUI.selectedRaidID = nil
	GUI.ActiveFrame = "NONE"
	GUI:Refresh()
	GUI:SetupTabInfo()
	MainWindow.lower_label:SetText(L["info.noraidsselected"])
end

local function EndActiveRaid()
	local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
	local activityTime = HeadCount:NewInstance("Time",utcDateTimeInSeconds)
	HeadCount:GetModule("RaidTracker"):endCurrentRaid(activityTime)
	GUI:Refresh(false)

	GUI.selectedRaid = HeadCount:GetModule("RaidTracker").raidListWrapper:retrieveMostRecentRaid()
	GUI.selectedRaidID = GUI.selectedRaid:retrieveStartingTime().epoc

	if GUI.ActiveFrame == "PLAYER" then
		GUI:ShowPlayersInRaid()
	end
	if GUI.ActiveFrame == "LOOT" then
		GUI:ShowLootInRaid()
	end
	if GUI.ActiveFrame == "BOSS" then
		GUI:ShowBossListInRaid()
	end
	if GUI.ActiveFrame == "WAIT" then
		GUI:ShowWaitListInRaid()
	end
end

local function ShowExportWindow(data)
	local f = AGU:Create("Frame")
	local control = AGU:Create("MultiLineEditBox")
	f:ReleaseChildren()
	f:SetCallback("OnClose", function(widget, event) control.button:Show(); AGU:Release(widget) end)
	f:SetTitle(L["gui.export.title"])
	f:SetLayout("fill")
	--f = scroll
	control:SetHeight(300)
	control:SetText(data)
	control.width = "fill"
	f:AddChild(control)
	f:PerformLayout()
	f:SetWidth(600)
	f:SetHeight(300)
	f:Show()
	control.button:Hide()
	if not IsMacClient() then
		-- Windows or Linux
		f.statustext:SetText(L["gui.win.export.tooltip"])
	else
		f.statustext:SetText(L["gui.mac.export.tooltip"])
	end
	f.frame:SetFrameStrata("TOOLTIP")
end

function GUI:ShowPlayersInRaid()
	if GUI.selectedRaid and MainWindow then
		GUI.ActiveFrame = "PLAYER"
		local raid = GUI.selectedRaid
		GUI.LowerBox:ReleaseChildren()
		local sort = AGU:Create("HeadCountHeaderItem")
		sort:SetColumn(1,"","",30,"")
		sort:SetColumn(2,L["gui.header.name"],"Name",100,string.format(L["gui.sort.header"],L["gui.header.name"]))
		sort:SetColumn(3,L["gui.header.start"],"Start",150,string.format(L["gui.sort.header"],L["gui.header.start"]))
		sort:SetColumn(4,L["gui.header.end"],"End",150,string.format(L["gui.sort.header"],L["gui.header.end"]))
		sort:SetColumn(5,L["gui.header.total"],"Total",130,string.format(L["gui.sort.header"],L["gui.header.total"]))
		sort:SetCallback("SortChosen",function(widget,event,colName)
			MainWindow.memberSort = colName
			MainWindow.sortDir = not MainWindow.sortDir
			GUI:ShowPlayersInRaid()
		end)
		sort:SetRelativeWidth(1)
		GUI.LowerBox:AddChild(sort)
		local numberOfPlayers = raid:GetNumberOfPlayers()
		local orderedPlayerList = raid:retrieveOrderedPlayerList(HeadCount:getRAID_MEMBER_SORT()[MainWindow.memberSort], true, true, true, false, false, MainWindow.sortDir)
		MainWindow.lower_label:SetText(string.format(L["gui.player.panel"],#orderedPlayerList))
		for k=1,#orderedPlayerList do
			local player = orderedPlayerList[k]
			local idLabel =  AGU:Create("Label")
			idLabel:SetText("    "..k)
			idLabel:SetWidth(40)

			GUI.LowerBox:AddChild(idLabel)

			local nLabel = AGU:Create("InteractiveLabel")
			nLabel:SetText(player.name)
			nLabel:SetWidth(100)

			local loot = raid:getLootListByPlayer(player.name)
			local pi = HeadCount:NewInstance("PlayerInformation",{ ["player"] = player, ["lootList"] = loot})
			nLabel:SetCallback("OnLeave",HideTooltip)
			local list, lines = pi:retrieveInformationList()
			nLabel:SetUserData("tooltip.multiline.data", list)
			nLabel:SetCallback("OnEnter",ShowMultilineTooltip)


			GUI.LowerBox:AddChild(nLabel)

			local sT = AGU:Create("Label")
			sT.label:SetJustifyH("LEFT")
			sT:SetText(HeadCount:getDateTimeAsString(player:retrieveStartingTime()))
			sT:SetWidth(150)
			GUI.LowerBox:AddChild(sT)

			local eT = AGU:Create("Label")
			eT.label:SetJustifyH("LEFT")
			eT:SetText(HeadCount:getDateTimeAsString(player:retrieveEndingTime()))
			eT:SetWidth(150)
			GUI.LowerBox:AddChild(eT)

			local tT = AGU:Create("Label")
			tT.label:SetJustifyH("LEFT")
			tT:SetText(HeadCount:getSecondsAsString(player:getTotalTime()))
			tT:SetWidth(100)
			GUI.LowerBox:AddChild(tT)

			local deleteButton = AGU:Create("InteractiveLabel")
			deleteButton:SetImage("Interface\\Buttons\\CancelButton-Up")
			deleteButton:SetImageSize(24,24)
			deleteButton:SetWidth(24)
			deleteButton:SetHeight(24)

			deleteButton:SetCallback("OnClick", function()
					GUI:ConfirmChoice(string.format(L["info.remove.attendee"],player.name),function()
						raid:removePlayer(player.name)
						GUI:ShowPlayersInRaid()
					end)
			end)

			local playerTT = {}
			table.insert(playerTT,"|cFF9999FF"..L["Remove member"] .. " " .. k.."|r")
			table.insert(playerTT,"|cffffffff"..player.name.."|r")
			table.insert(playerTT,"|cffffffff"..HeadCount:getDateTimeAsString(raid:retrieveStartingTime()).."|r")
			if raid.zone then
				table.insert(playerTT,raid.zone)
			end
			local difficulty = raid.difficulty or 1
			local difficultyString = _G["RAID_DIFFICULTY"..difficulty]
			table.insert(playerTT,"|cffffffffDifficulty:"..difficultyString.."|r")
			SetupMultilineTooltip(deleteButton,playerTT)
			GUI.LowerBox:AddChild(deleteButton)

		end
	end
end

function GUI:ShowBossListInRaid()
	if GUI.selectedRaid and MainWindow then
		GUI.ActiveFrame = "BOSS"
		local raid = GUI.selectedRaid
		GUI.LowerBox:ReleaseChildren()
		local sort = AGU:Create("HeadCountHeaderItem")
		sort:SetColumn(1,"","",30,"")
		sort:SetColumn(2,"Time","",150,"")
		sort:SetColumn(3,"Members","",100,L["gui.boss.member.tooltip"])
		sort:SetColumn(4,"Name","",150,"")
		sort:SetRelativeWidth(1)
		GUI.LowerBox:AddChild(sort)
		local orderedBossList = raid:retrieveOrderedBossList()
		MainWindow.lower_label:SetText(string.format(L["gui.boss.panel"],#orderedBossList))
		for k=1,#orderedBossList do
			local boss = orderedBossList[k]
			local idLabel =  AGU:Create("Label")
			idLabel:SetText("    "..k)
			idLabel:SetWidth(40)

			GUI.LowerBox:AddChild(idLabel)

			local tLabel = AGU:Create("Label")
			tLabel.label:SetJustifyH("LEFT")
			tLabel:SetText(HeadCount:getDateTimeAsString(boss.runtime))
			tLabel:SetWidth(150)
			GUI.LowerBox:AddChild(tLabel)

			local memberLabel = AGU:Create("InteractiveLabel")
			memberLabel:SetText(#boss.players)
			memberLabel:SetWidth(100)
			local list = {}

			table.insert(list, "|cFF9999FF" .. L["Members"] .. "|r")
			for k,v in pairs(boss.players) do
				table.insert(list,v)
			end

			memberLabel:SetUserData("tooltip.multiline.data", list)
			memberLabel:SetCallback("OnEnter",ShowMultilineTooltip)
			memberLabel:SetCallback("OnLeave",HideTooltip)

			GUI.LowerBox:AddChild(memberLabel)

			local nameLabel = AGU:Create("Label")
			nameLabel.label:SetJustifyH("LEFT")
			nameLabel:SetText(boss.name)
			nameLabel:SetWidth(250)
			GUI.LowerBox:AddChild(nameLabel)

			local deleteButton = AGU:Create("InteractiveLabel")
			deleteButton:SetImage("Interface\\Buttons\\CancelButton-Up")
			deleteButton:SetImageSize(24,24)
			deleteButton:SetWidth(24)
			deleteButton:SetHeight(24)

			deleteButton:SetCallback("OnClick", function()
					GUI:ConfirmChoice(string.format(L["info.remove.boss"],boss.name),function()
						raid:removeBoss(boss.name)
						GUI:ShowBossListInRaid()
					end)
			end)
			local bossTT = {}
			table.insert(bossTT,"|cFF9999FF"..L["Remove boss"] .. " " .. k.."|r")
			table.insert(bossTT,"|cffffffff"..boss.name.."|r")
			table.insert(bossTT,"|cffffffff"..HeadCount:getDateTimeAsString(raid:retrieveStartingTime()).."|r")
			if raid.zone then
				table.insert(bossTT,raid.zone)
			end
			local difficulty = boss.difficulty or 1
			local difficultyString = _G["RAID_DIFFICULTY"..difficulty]
			table.insert(bossTT,"|cffffffffDifficulty:"..difficultyString.."|r")
			SetupMultilineTooltip(deleteButton,bossTT)
			GUI.LowerBox:AddChild(deleteButton)
		end
	end
end

function GUI:ShowLootInRaid()
	if GUI.selectedRaid and MainWindow then
		GUI.ActiveFrame = "LOOT"
		local raid = GUI.selectedRaid
		GUI.LowerBox:ReleaseChildren()
		local lootList = raid.lootList
		MainWindow.lower_label:SetText(string.format(L["gui.loot.panel"],#lootList))
		for k=1,#lootList do
			local loot = lootList[k]

			local idLabel =  AGU:Create("Label")
			idLabel.label:SetJustifyH("LEFT")
			idLabel:SetText("    "..k)
			idLabel:SetWidth(40)

			GUI.LowerBox:AddChild(idLabel)

			local tLabel = AGU:Create("InteractiveLabel")
			tLabel:SetImage(loot.texture)
			tLabel:SetImageSize(34,34)
			tLabel:SetWidth(240)
			tLabel:SetText("|n"..loot.link)
			--tLabel.label:SetJustifyV("MIDDLE")
			SetupTooltip(tLabel,L["info.loot.link"])
			tLabel:SetCallback("OnClick", function()
				local itemName, itemLink, itemQuality, itemLevel, itemMinLevel, itemType, itemSubType, itemCount, itemEquipLoc, itemTexture = GetItemInfo(loot.itemId)
				if (not itemName) then
					-- item info is not returning, query the server
					GameTooltip:SetHyperlink("item:".. loot.itemId ..":0:0:0:0:0:0:0")
				else
					ChatEdit_InsertLink(itemLink)
				end
			end)
			GUI.LowerBox:AddChild(tLabel)

			local memberLabel = AGU:Create("Label")
			memberLabel.label:SetJustifyH("LEFT")
			memberLabel:SetText(L["Looted by"] .. " " .. loot.player .. " (" .. HeadCount:getDateTimeAsString(loot.looted)  .. ")")
			memberLabel:SetWidth(200)
			GUI.LowerBox:AddChild(memberLabel)

			local nameLabel = AGU:Create("Label")
			nameLabel:SetText(" ")
			nameLabel:SetWidth(50)
			GUI.LowerBox:AddChild(nameLabel)


			local managementButton = AGU:Create("InteractiveLabel")
			managementButton:SetImage("Interface\\Buttons\\UI-GuildButton-PublicNote-Up")
			managementButton:SetImageSize(13,13)
			managementButton:SetWidth(13)
			managementButton:SetHeight(13)

			managementButton:SetCallback("OnClick", function()
				GUI:ShowLootPopup(GUI.selectedRaidID,k)
			end)
			local manageTT = {}

			table.insert(manageTT,"|cFF9999FF"..L["Manage loot"].."|r")
			table.insert(manageTT,loot.link)
			table.insert(manageTT,"|cFFffffff"..L["Looted by"]..": "..loot.player.."|r")
			local lootSource = loot.source
			if lootSource and strlen(lootSource) > 0 then
				table.insert(manageTT,"|cFFFFFFFF"..L["Source"]..": "..loot.source.."|r")
			else
				table.insert(manageTT,"|cFFFFFFFF"..L["Source"]..": "..L["Trash mob"].."|r")
			end
			table.insert(manageTT,"|cFFFFFFFF"..L["Loot cost"]..": "..loot.cost.."|r")
			if loot.note and strlen(loot.note) > 0 then
				table.insert(manageTT,"|cFFFFFFFF"..L["Note"]..": "..loot.note.."|r")
			end
			SetupMultilineTooltip(managementButton,manageTT)
			GUI.LowerBox:AddChild(managementButton)

			local deleteButton = AGU:Create("InteractiveLabel")
			deleteButton:SetImage("Interface\\Buttons\\CancelButton-Up")
			deleteButton:SetImageSize(24,24)
			deleteButton:SetWidth(24)
			deleteButton:SetHeight(24)

			deleteButton:SetCallback("OnClick", function()
					GUI:ConfirmChoice(string.format(L["info.remove.loot"],loot.name),function()
						raid:removeLoot(k)
						GUI:ShowLootInRaid()
					end)
			end)

			local deleteTT = {}
			table.insert(deleteTT,"|cFF9999FF"..L["Remove loot"].."|r")
			table.insert(deleteTT,loot.link)
			table.insert(deleteTT,"|cFFffffff"..L["Looted by"]..": "..loot.player.."|r")
			local lootSource = loot.source
			if lootSource and strlen(lootSource) > 0 then
				table.insert(deleteTT,"|cFFFFFFFF"..L["Source"]..": "..loot.source.."|r")
			else
				table.insert(deleteTT,"|cFFFFFFFF"..L["Source"]..": "..L["Trash mob"].."|r")
			end
			table.insert(deleteTT,"|cFFFFFFFF"..L["Loot cost"]..": "..loot.cost.."|r")
			if loot.note and strlen(loot.note) > 0 then
				table.insert(deleteTT,"|cFFFFFFFF"..L["Note"]..": "..loot.note.."|r")
			end
			SetupMultilineTooltip(deleteButton,deleteTT)
			GUI.LowerBox:AddChild(deleteButton)
		end
	end
end

function GUI:ShowAttendanceList()
	if MainWindow then
		GUI.ActiveFrame = "WAIT"
		local raid = GUI.selectedRaid
		GUI.LowerBox:ReleaseChildren()
		local sort = AGU:Create("HeadCountHeaderItem")
		sort:SetColumn(1,"","",30,"")
		sort:SetColumn(2,L["gui.header.name"],"Name",100,string.format(L["gui.sort.header"],L["gui.header.name"]))
		sort:SetColumn(3,L["gui.header.thirty"],"Thirty",100,string.format(L["gui.sort.header"],L["gui.header.thirty"]))
		sort:SetColumn(4,L["gui.header.sixty"],"Sixety",100,string.format(L["gui.sort.header"],L["gui.header.sixty"]))
		sort:SetColumn(5,L["gui.header.ninety"],"Ninety",100,string.format(L["gui.sort.header"],L["gui.header.ninety"]))
		sort:SetColumn(6,L["gui.header.totaldays"],"Total",105,string.format(L["gui.sort.header"],L["gui.header.totaldays"]))

		sort:SetCallback("SortChosen",function(widget,event,colName)
			MainWindow.attendSort = colName
			MainWindow.sortDir = not MainWindow.sortDir
			GUI:ShowAttendanceList()
		end)
		sort:SetRelativeWidth(1)
		GUI.LowerBox:AddChild(sort)
		local orderedPlayerList = HeadCount:GetModule("RaidTracker"):retrieveAttendanceTotals(MainWindow.attendSort, MainWindow.sortDir)
		MainWindow.lower_label:SetText(string.format(L["gui.attendance.panel"],#orderedPlayerList))
		for k=1,#orderedPlayerList do
			local player = orderedPlayerList[k]
			local idLabel =  AGU:Create("Label")
			idLabel:SetText("    "..k)
			idLabel:SetWidth(40)
			GUI.LowerBox:AddChild(idLabel)

			local nLabel = AGU:Create("Label")
			nLabel.label:SetJustifyH("LEFT")
			nLabel:SetText(player.name)
			nLabel:SetWidth(100)
			GUI.LowerBox:AddChild(nLabel)

			local tLabel = AGU:Create("Label")
			tLabel.label:SetJustifyH("LEFT")
			if player.thirty == -1 then
				tLabel:SetText(L["gui.outof.range"])
			else
				tLabel:SetText(string.format("%u%%",player.thirty))
			end
			tLabel:SetWidth(100)
			GUI.LowerBox:AddChild(tLabel)

			local sLabel = AGU:Create("Label")
			sLabel.label:SetJustifyH("LEFT")
			if player.sixty == -1 then
				sLabel:SetText(L["gui.outof.range"])
			else
				sLabel:SetText(string.format("%u%%",player.sixty))
			end
			sLabel:SetWidth(100)
			GUI.LowerBox:AddChild(sLabel)

			local iLabel = AGU:Create("Label")
			iLabel.label:SetJustifyH("LEFT")
			if player.ninety == -1 then
				iLabel:SetText(L["gui.outof.range"])
			else
				iLabel:SetText(string.format("%u%%",player.ninety))
			end
			iLabel:SetWidth(100)
			GUI.LowerBox:AddChild(iLabel)

			local lLabel = AGU:Create("Label")
			lLabel.label:SetJustifyH("LEFT")
			if player.total == -1 then
				lLabel:SetText(L["gui.outof.range"])
			else
				lLabel:SetText(string.format("%u%%",player.total))
			end
			lLabel:SetWidth(110)
			GUI.LowerBox:AddChild(lLabel)

		end
	end
end

function GUI:ShowWaitListInRaid()
	if GUI.selectedRaid and MainWindow then
		GUI.ActiveFrame = "WAIT"
		local raid = GUI.selectedRaid
		GUI.LowerBox:ReleaseChildren()
		local sort = AGU:Create("HeadCountHeaderItem")
		sort:SetColumn(1,"","",30,"")
		sort:SetColumn(2,L["gui.header.name"],"Name",100,string.format(L["gui.sort.header"],L["gui.header.name"]))
		sort:SetColumn(3,L["gui.header.waittime"],"Waitlist",150,string.format(L["gui.sort.header"],L["gui.header.waittime"]))
		sort:SetColumn(4,L["gui.header.waitnote"],"WaitlistNote",150,string.format(L["gui.sort.header"],L["gui.header.waitnote"]))
		sort:SetCallback("SortChosen",function(widget,event,colName)
			MainWindow.waitlistSort = colName
			MainWindow.sortDir = not MainWindow.sortDir
			GUI:ShowWaitListInRaid()
		end)
		sort:SetRelativeWidth(1)
		GUI.LowerBox:AddChild(sort)
		-- populate based on input data
		-- we need Number, Name, Time, Note and Delete
		local orderedPlayerList = raid:retrieveOrderedPlayerList(HeadCount:getRAID_MEMBER_SORT()[MainWindow.waitlistSort], false, false, false, true, false, MainWindow.sortDir)
		MainWindow.lower_label:SetText(string.format(L["gui.waitlist.panel"],#orderedPlayerList))
		for k=1,#orderedPlayerList do
			local player = orderedPlayerList[k]
			local idLabel =  AGU:Create("Label")
			idLabel:SetText("    "..k)
			idLabel:SetWidth(40)

			GUI.LowerBox:AddChild(idLabel)

			local nLabel = AGU:Create("Label")
			nLabel.label:SetJustifyH("LEFT")
			nLabel:SetText(player.name)
			nLabel:SetWidth(100)
			GUI.LowerBox:AddChild(nLabel)

			local sT = AGU:Create("Label")
			sT.label:SetJustifyH("LEFT")
			sT:SetText(HeadCount:getDateTimeAsString(player.waitlistActivityTime))
			sT:SetWidth(150)
			GUI.LowerBox:AddChild(sT)

			local eT = AGU:Create("Label")
			eT.label:SetJustifyH("LEFT")
			eT:SetText(player.waitlistNote)
			eT:SetWidth(250)
			GUI.LowerBox:AddChild(eT)

			local deleteButton = AGU:Create("InteractiveLabel")
			deleteButton:SetImage("Interface\\Buttons\\CancelButton-Up")
			deleteButton:SetImageSize(24,24)
			deleteButton:SetWidth(24)
			deleteButton:SetHeight(24)

			deleteButton:SetCallback("OnClick", function()
					GUI:ConfirmChoice(string.format(L["info.remove.attendee"],player.name),function()
						raid:removeWaitlistPlayer(player.name)
						GUI:ShowWaitListInRaid()
					end)
			end)
			local waitTT = {}
			table.insert(waitTT,"|cFF9999FF"..L["Remove wait list member"] .. " " .. k.."|r")
			table.insert(waitTT,"|cffffffff"..player.name.."|r")
			if raid.zone then
				table.insert(waitTT,raid.zone)
			end
			SetupMultilineTooltip(deleteButton,waitTT)
			GUI.LowerBox:AddChild(deleteButton)
		end
	end
end

function GUI:PopulateRaidList(initial)
	if MainWindow and self.UpperBox then
		local rlW = HeadCount:GetModule("RaidTracker").raidListWrapper
		self.UpperBox:ReleaseChildren()
		self.LowerBox:ReleaseChildren()
		local orderedRaidList = rlW:retrieveOrderedRaidList(false)
		for k=1,#orderedRaidList do
			local v = orderedRaidList[k]
			-- TODO Make a Widget to show a Given Row so I can do Highlighting and Such
			local zone = v.zone
			local difficulty = v.difficulty
			local difficultyString = ""
			if difficulty then
				difficultyString = _G["RAID_DIFFICULTY"..difficulty]
			end
			if zone then
				if difficulty then
					zone = zone .. " (" .. difficultyString .. ")"
				else
					zone = zone
				end
			end
			-- Add active label to zone
			if not v.isFinalized and HeadCount:GetModule("RaidTracker").isCurrentRaidActive then
				-- raid is not finalized and raid is active
				if zone then
					zone = zone .. " (" .. L["Active"] .. ")"
				else
					zone = "(" .. L["Active"] .. ")"
				end
			end
			if not zone then
				zone = UNKNOWN
			end
			local raidInfoItem = AGU:Create("HeadCountRaidItem")
			raidInfoItem:SetFullWidth(true)
			raidInfoItem:SetNumber(k)
			raidInfoItem:SetDate(HeadCount:getDateTimeAsString(v:retrieveStartingTime()))
			raidInfoItem:SetZone(zone)
			raidInfoItem:SetRaidID(v:retrieveStartingTime().epoc)

			local deleteTT = {}
			table.insert(deleteTT,"|cFF9999FF"..string.format(L["gui.remove.raid"],k).."|r")
			table.insert(deleteTT,"|cffffffff" ..HeadCount:getDateTimeAsString(v:retrieveStartingTime()).. "|r")
			table.insert(deleteTT,zone)

			local exportTT = {}
			table.insert(exportTT,"|cFF9999FF"..string.format(L["gui.export.raid"],k).."|r")
			table.insert(exportTT,"|cffffffff" ..HeadCount:getDateTimeAsString(v:retrieveStartingTime()).. "|r")
			table.insert(exportTT,zone)


			raidInfoItem.export.tt = exportTT
			raidInfoItem.delete.tt = deleteTT

			raidInfoItem:SetCallback("RaidDeleted", function (widget, event, id)
				GUI:ConfirmChoice(L["info.remove.raid"],function()
					HeadCount:GetModule("RaidTracker"):removeRaid(id)
					GUI:Refresh()
				end)
			end)

			raidInfoItem:SetCallback("RaidExported", function (widget, event, id)
				local raid = HeadCount:GetModule("RaidTracker").raidListWrapper:getRaidById(id)
				local data = HeadCount:exportRaid(raid)
				ShowExportWindow(data)
			end)

			raidInfoItem:SetCallback("RaidSelected", function(widget,event,id, row)
				for k,v in ipairs(GUI.UpperBox.children) do
					v:SetSelected(false)
					if k == row then
						v:SetSelected(true)
					end
				end
				local raid = HeadCount:GetModule("RaidTracker").raidListWrapper:getRaidById(id)
				GUI.selectedRaid = raid
				GUI.selectedRaidID = id
				GUI:ShowPlayersInRaid()
				self:SetupTabInfo()
			end)
			if initial and k == 1 then
				GUI.selectedRaid = v
				GUI.selectedRaidID = v:retrieveStartingTime().epoc
				raidInfoItem:SetSelected(true)
				self:SetupTabInfo()
			end
			self.UpperBox:AddChild(raidInfoItem)
		end
		if initial and #orderedRaidList > 0 then
			self:ShowPlayersInRaid()
		end
	end
end

function GUI:SetupTabInfo()
	local raid = GUI.selectedRaid
	if MainWindow then
		if raid then
			local difficulty = raid.difficulty or 1
			local difficultyString = _G["RAID_DIFFICULTY"..difficulty]
			local raidTime = "|cFFffffff" .. HeadCount:getDateTimeAsString(raid:retrieveStartingTime()) .. "|r"
			local memberTT = {}
			local waitTT = {}
			local bossTT = {}
			local lootTT = {}
			table.insert(memberTT,"|cFF9999FF"..L["gui.player.list"].."|r")
			table.insert(memberTT,raidTime)
			table.insert(waitTT,"|cFF9999FF"..L["gui.wait.list"].."|r")
			table.insert(waitTT,raidTime)
			table.insert(bossTT,"|cFF9999FF"..L["gui.boss.list"].."|r")
			table.insert(bossTT,raidTime)
			table.insert(lootTT,"|cFF9999FF"..L["gui.loot.list"].."|r")
			table.insert(lootTT,raidTime)
			if raid.zone then
				table.insert(memberTT,raid.zone)
				table.insert(waitTT,raid.zone)
				table.insert(bossTT,raid.zone)
				table.insert(lootTT,raid.zone)
			end
			if raid.difficulty then
				table.insert(memberTT,"|cFFffffff".. L["Difficulty"] .. ": " .. difficultyString.."|r")
				table.insert(waitTT,"|cFFffffff" .. L["Difficulty"] .. ": " .. difficultyString.."|r")
				table.insert(bossTT,"|cFFffffff" .. L["Difficulty"] .. ": " .. difficultyString.."|r")
				table.insert(lootTT,"|cFFffffff" .. L["Difficulty"] .. ": " .. difficultyString.."|r")
			end
			table.insert(memberTT,"|cFFffffff" .. L["Number of members"] .. ": " .. raid.numberOfPlayers .. "|r")
			table.insert(waitTT,"|cFFffffff" .. L["Number of members"] .. ": " .. raid:getNumberOfWaitlistPlayers() .. "|r")
			table.insert(bossTT,"|cFFffffff" .. L["Number of bosses"] .. ": " .. raid.numberOfBosses .. "|r")
			table.insert(lootTT,"|cFFffffff" .. L["Number of items"] .. ": " .. #raid.lootList .. "|r")
			SetupMultilineTooltip(MainWindow.memberButton,memberTT)
			SetupMultilineTooltip(MainWindow.bossButton,bossTT)
			SetupMultilineTooltip(MainWindow.waitButton,waitTT)
			SetupMultilineTooltip(MainWindow.lootButton,lootTT)
			MainWindow.memberButton.frame:Show()
			MainWindow.bossButton.frame:Show()
			MainWindow.waitButton.frame:Show()
			MainWindow.lootButton.frame:Show()
			MainWindow.attentButton.frame:Show()
		else
			MainWindow.memberButton.frame:Hide()
			MainWindow.bossButton.frame:Hide()
			MainWindow.waitButton.frame:Hide()
			MainWindow.lootButton.frame:Hide()
			MainWindow.attentButton.frame:Hide()
		end
	end
end

function GUI:ShowTracker()
	if MainWindow then return end
	GUI.ActiveFrame = "NONE"
	MainWindow = AGU:Create("Window")
	MainWindow:SetTitle(L["gui.window.title"])
	MainWindow:SetWidth(620)
	MainWindow:SetHeight(420)
	MainWindow:SetCallback("OnClose", function(widget)
		widget:ReleaseChildren()
		widget:Release()
		MainWindow.history_label = nil
		MainWindow.lower_label = nil
		MainWindow.memberButton = nil
		MainWindow.bossButton = nil
		MainWindow.waitButton = nil
		MainWindow.lootButton = nil
		GUI.UpperBox = nil
		GUI.LowerBox = nil
		MainWindow = nil
		if self.annouce_window then
			AGU:Release(self.annouce_window)
			self.annouce_window = nil
		end
		if self.lootwindow then
			AGU:Release(self.lootwindow)
			self.lootwindow = nil
		end
		GUI.ActiveFrame = "NONE"
	end)
	MainWindow:SetLayout("Flow")
	MainWindow:EnableResize(false)
	-- Create upper Box
	-- UpperLabel
	local raidCount = HeadCount.db.profile.raidListWrapper.numberOfRaids
	local raidTitle = AGU:Create("Label")
	raidTitle:SetText(string.format(L["gui.history.title"],raidCount))
	raidTitle:SetColor(0.63,0.63,1.0,1.0)
	raidTitle:SetRelativeWidth(.90)
	MainWindow:AddChild(raidTitle)
	MainWindow.history_label = raidTitle

	-- Annouce Icon
	local raidAnnounce = AGU:Create("InteractiveLabel")
	raidAnnounce:SetImage("Interface\\Buttons\\UI-GuildButton-MOTD-Up")
	raidAnnounce:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	raidAnnounce:SetImageSize(13,13)
	raidAnnounce:SetRelativeWidth(0.03)
	SetupTooltip(raidAnnounce,L["gui.annouce.title"])
	raidAnnounce:SetCallback("OnClick", GUI.ShowAnnounceWindow)
	MainWindow:AddChild(raidAnnounce)


	-- End Raid
	local raidEnd = AGU:Create("InteractiveLabel")
	raidEnd:SetImage("Interface\\Buttons\\UI-GroupLoot-Pass-Up")
	raidEnd:SetHighlight("Interface\\Buttons\\UI-GroupLoot-Pass-Highlight")
	raidEnd:SetImageSize(13,13)
	raidEnd:SetRelativeWidth(0.03)
	SetupTooltip(raidEnd,L["gui.end.active.raid"])
	raidEnd:SetCallback("OnClick", function()
		local currentRaidId = HeadCount:GetModule("RaidTracker").raidListWrapper:retrieveMostRecentRaidId()
		if HeadCount:GetModule("RaidTracker"):isRaidActive() then
			GUI:ConfirmChoice(L["gui.confirm.end.raid"],EndActiveRaid)
		else
			HeadCount:LogInformation(string.format(L["info.end.raid.noraidsexists"], HeadCount.TITLE, HeadCount.VERSION))
		end
	end)
	MainWindow:AddChild(raidEnd)

	-- Delete all raids
	local raidWipe = AGU:Create("InteractiveLabel")
	raidWipe:SetImage("Interface\\CharacterFrame\\Disconnect-Icon")
	raidWipe:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	raidWipe:SetImageSize(24,24)
	raidWipe:SetRelativeWidth(0.03)
	SetupTooltip(raidWipe,L["gui.remove.all.raids"])
	raidWipe:SetCallback("OnClick", function()
		if HeadCount.db.profile.raidListWrapper.numberOfRaids > 0 then
			GUI:ConfirmChoice(L["gui.confirm.remove.all"],RemoveAllRaids)
		end
	end)
	MainWindow:AddChild(raidWipe)

	-- Upper Box simple groupwith our border
	local UpperBox = AGU:Create("SimpleGroup")
	UpperBox:SetHeight(150)
	UpperBox:SetFullWidth(true)
	UpperBox:SetLayout("Fill")

	local UBScroll = AGU:Create("ScrollFrame")

	UpperBox:AddChild(UBScroll)

	local border = CreateFrame("Frame", nil, UpperBox.content)
	border:SetPoint("TOPLEFT", 0, 0)
	border:SetPoint("BOTTOMRIGHT", -1, 3)
	border:SetBackdrop(SimpleGroupBackdrop)
	border:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	border:SetBackdropBorderColor(0.4, 0.4, 0.4)
	UBScroll:SetHeight(150)
	MainWindow:AddChild(UpperBox)
	UBScroll:SetLayout("Flow")
	GUI.UpperBox = UBScroll

	-- Create Lower Box
	local lowerTitle = AGU:Create("Label")
	lowerTitle:SetText("")
	lowerTitle:SetColor(0.63,0.63,1.0,1.0)
	lowerTitle:SetRelativeWidth(.84)
	MainWindow:AddChild(lowerTitle)
	MainWindow.lower_label = lowerTitle
	MainWindow.lower_label:SetText(L["info.noraidsselected"])

	local memberButton = AGU:Create("InteractiveLabel")
	memberButton:SetImage("Interface\\CharacterFrame\\UI-StateIcon")
	memberButton.image:SetTexCoord(0.5,1.0,0.0,0.5)
	memberButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	memberButton:SetImageSize(18,18)
	memberButton:SetRelativeWidth(0.03)
	memberButton:SetCallback("OnClick",GUI.ShowPlayersInRaid)
	MainWindow.memberSort = "Start"
	MainWindow:AddChild(memberButton)
	MainWindow.memberButton = memberButton

	local waitlistButton = AGU:Create("InteractiveLabel")
	waitlistButton:SetImage("Interface\\GossipFrame\\PetitionGossipIcon")
	waitlistButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	waitlistButton:SetImageSize(13,13)
	waitlistButton:SetRelativeWidth(0.03)
	waitlistButton:SetCallback("OnClick", GUI.ShowWaitListInRaid)
	MainWindow:AddChild(waitlistButton)
	MainWindow.waitButton = waitlistButton
	MainWindow.waitlistSort = "Waitlist"

	local bossButton = AGU:Create("InteractiveLabel")
	bossButton:SetImage("Interface\\TargetingFrame\\UI-TargetingFrame-Skull")
	bossButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	bossButton:SetImageSize(14,11)
	bossButton:SetRelativeWidth(0.03)
	bossButton:SetCallback("OnClick", GUI.ShowBossListInRaid)
	MainWindow:AddChild(bossButton)
	MainWindow.bossButton = bossButton

	local lootButton = AGU:Create("InteractiveLabel")
	lootButton:SetImage("Interface\\Buttons\\UI-GroupLoot-Coin-Up")
	lootButton:SetHighlight("Interface\\Buttons\\UI-GroupLoot-Coin-Highlight")
	lootButton:SetImageSize(14,14)
	lootButton:SetRelativeWidth(0.03)
	lootButton:SetCallback("OnClick", GUI.ShowLootInRaid)
	MainWindow:AddChild(lootButton)
	MainWindow.lootButton = lootButton


	local attendButton = AGU:Create("InteractiveLabel")
	attendButton:SetImage("Interface\\Calendar\\MeetingIcon")
	attendButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	attendButton:SetImageSize(14,14)
	attendButton:SetRelativeWidth(0.03)
	local tt = {}
	table.insert(tt,"|cFF9999FF"..L["gui.attendance.summary"].."|r")
	table.insert(tt,L["gui.attendance.desc"])
	SetupMultilineTooltip(attendButton,tt)
	attendButton:SetCallback("OnClick", GUI.ShowAttendanceList)
	MainWindow.attendSort = "Name"
	MainWindow:AddChild(attendButton)
	MainWindow.attentButton = attendButton

	-- Make Lower Label

	local LowerBox = AGU:Create("SimpleGroup")
	LowerBox:SetHeight(165)
	LowerBox:SetFullWidth(true)
	LowerBox:SetLayout("Fill")

	local LBScroll = AGU:Create("ScrollFrame")
	LBScroll:SetHeight(135)
	LowerBox:AddChild(LBScroll)

	local lborder = CreateFrame("Frame", nil, LowerBox.content)
	lborder:SetPoint("TOPLEFT", 0, 5)
	lborder:SetPoint("BOTTOMRIGHT", 3, -5)
	lborder:SetBackdrop(SimpleGroupBackdrop)
	lborder:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	lborder:SetBackdropBorderColor(0.4, 0.4, 0.4)
	LBScroll:SetLayout("Flow")
	GUI.LowerBox = LBScroll

	MainWindow:AddChild(LowerBox)
	MainWindow.sortDir = true
	GUI:PopulateRaidList(true)
	self:SetupTabInfo()
end

function GUI:ConfirmChoice(message,callback)
	if not StaticPopupDialogs["HEADCOUNT_CONFIRM"] then
		StaticPopupDialogs["HEADCOUNT_CONFIRM"] = {
			text = "",
			button1 = ACCEPT,
			button2 = CANCEL,
			OnAccept = function()
				return false
			end,
			timeout = 0,
			whileDead = 1,
			hideOnEscape = 1,
			exclusive= true,
		}
	end
	StaticPopupDialogs["HEADCOUNT_CONFIRM"].text = message
	StaticPopupDialogs["HEADCOUNT_CONFIRM"].OnAccept = callback
	StaticPopup_Show("HEADCOUNT_CONFIRM")
end

function GUI:ShowAnnounceWindow()
	if GUI.annouce_window then return end
	self = GUI
	local frame = AGU:Create("HeadCountDialog")
	frame:SetTitle(L["gui.annouce.title"])
	frame:SetWidth(400)
	frame:SetHeight(200)
	frame:SetCallback("OnClose", function(widget)
		widget:ReleaseChildren()
		widget:Release()
		GUI.annouce_window = nil
	end)

	frame:SetCallback("OnAccept", function(widget)
		local choice = frame:GetUserData("type-choice")
		local channel = frame:GetUserData("channel-choice")
		widget:ReleaseChildren()
		widget:Release()
		GUI.annouce_window = nil
		if choice == 1 then
			HeadCount:GetModule("RaidTracker"):announceWaitlist(channel)
		else
			HeadCount:GetModule("RaidTracker"):processListWaitlist(channel)
		end
	end)

	frame:SetAcceptLabel("Annouce")
	frame:SetCancelLabel("Cancel")
	frame:EnableHotKey()

	frame:SetLayout("Flow")
	self.annouce_window = frame
	frame:SetUserData("type-choice",1)
	frame:SetUserData("channel-choice","GUILD")

	local spacer1 = AGU:Create("Label")
	spacer1:SetRelativeWidth(0.22)
	local spacer2 = AGU:Create("Label")
	spacer2:SetRelativeWidth(0.22)
	local spacer3 = AGU:Create("Label")
	spacer3:SetRelativeWidth(0.2)

	-- Create first DropDown
	local methodDrop = AGU:Create("Dropdown")
	methodDrop:SetLabel("Type")
	methodDrop:SetList( { [1] = L["gui.announce.wait.list"], [2] = L["gui.list.wait.list"]})
	methodDrop:SetCallback("OnValueChanged", function(self,event, key,checked)
		frame:SetUserData("type-choice",key)
	end)
	methodDrop:SetValue(1)
	frame:AddChild(spacer1)
	frame:AddChild(methodDrop)
	frame:AddChild(spacer3)
	methodDrop.label:SetTextColor(0.63,0.63,1,1)
	-- Second Dropdown
	local channelDrop = AGU:Create("Dropdown")
	channelDrop:SetLabel("Channel")
	channelDrop:SetList(HeadCount.CHANNEL_LIST)
	channelDrop:SetCallback("OnValueChanged", function(self, event, key,checked)
		frame:SetUserData("channel-choice",key)
	end)
	channelDrop:SetValue("GUILD")
	channelDrop.label:SetTextColor(0.63,0.63,1,1)
	frame:AddChild(spacer2)
	frame:AddChild(channelDrop)
end